/*
 * common.c - shared system foundation functions
 *   by uratan! 2014.10.23
 */
#include "common.h"
#include <chip.h>

	/*
	 * functions prototpes are in common.h
	 */

/* =============================================================== */

/*
 * initial UART0
 */
void cm_init_uart(u_long baudrate, u_long mainCLK)
{
	u_long n, tmp;
	u_long Fdiv;

	/*
	 * enable clock for UART0
	 */
	LPC_SYSCTL->SYSAHBCLKCTRL |= (1 << SYSCTL_CLOCK_UART0);

	/*
	 * enable clock for baudrate generator
	 */
	LPC_SYSCTL->UARTCLKDIV = 1;		/* mainCLK(12MHz or 60MHz) thru */

	/*
	 * setup fractional divider (1+5/8 = 13/8)
	 */
	LPC_SYSCTL->UARTFRGDIV  = 8*32 - 1;	/* use this (0xff only) */
	LPC_SYSCTL->UARTFRGMULT = 5*32;

#if USE_DEBUG_OUTPUT
#else /* USE_DEBUG_OUTPUT */
	/*
	 * connect U0_TxD for PIO0_4, U0_RxD for PIO0_0 (same as ISP)
	 */
	n = 0;
	tmp = LPC_SWM->PINASSIGN[n];
	tmp &= ~( (0xff <<8) | (0xff <<0) );
	tmp |=  ( (  0  <<8) | (  4  <<0) );
	LPC_SWM->PINASSIGN[n] = tmp;
#endif /* USE_DEBUG_OUTPUT */

	/*
	 * setup uart0
	 *   use Fractional Divider for 115kbps@12MHz (12MHz*8/13)
	 *       (it works well when mainCLK=60MHz)
	 */
	LPC_USART0->CFG = 0x0005;	/* enable UART, 8bit, no-parity, 1stopbit */
	LPC_USART0->CTRL = 0x000;	/* no special operations */
	LPC_USART0->INTENCLR = 0xf965;		/* do not use interrupts */
	Fdiv = 8 * mainCLK / 16 / baudrate / 13;	/* calc division */
	LPC_USART0->BRG = Fdiv - 1;
		/*
		 * 8bit BRG covers:
		 *     Fdiv  :  256    ...    1
		 *   ------------------------------
		 *   at 12MHz: 1803bps ...  461kbps
		 *   at 30MHz: 4507bps ... 1154kbps
		 */
	/*
	 * finally, disable pull-up-R on output port (PIO0_4)
	 */
	LPC_SYSCTL->SYSAHBCLKCTRL |= (1 << SYSCTL_CLOCK_IOCON);

	n = IOCON_PIO4;
	tmp = LPC_IOCON->PIO0[n];
	tmp &= ~PIN_MODE_MASK;
	tmp |= (PIN_MODE_INACTIVE <<PIN_MODE_BITNUM);
	LPC_IOCON->PIO0[n] = tmp;

	LPC_SYSCTL->SYSAHBCLKCTRL &= ~(1 << SYSCTL_CLOCK_IOCON);
}

/*
 * write a char to UART0
 */
void cm_putchar(int ch)
{
	/*
	 * wait for the buffer empty
	 */
	while( !(LPC_USART0->STAT & UART_STAT_TXRDY) ) {
		;
	}

	LPC_USART0->TXDATA = ch & 0xff;
	return;
}

/*
 * check arrival of a char from UART0
 */
int cm_keyin(void)
{
	if(LPC_USART0->STAT & UART_STAT_RXRDY) {
		return(LPC_USART0->RXDATA & 0xff);		/* something has come */
	}

	return(-1);		/* nothing has come */
}

/*
 * wait until some char will come
 */
u_char cm_getchar(void)
{
	int i;

	while(1) {
		i = cm_keyin();
		if(i != -1) {
			break;		/* something arrived */
		}
	}

	return(i);
}

/* =============================================================== */
